package com.example.survey.service.impl;

import com.example.survey.dao.ProjectDao;
import com.example.survey.dao.RecordDao;
import com.example.survey.dao.RespondentDao;
import com.example.survey.dao.UserDao;
import com.example.survey.dto.record.ModifyRecordDTO;
import com.example.survey.dto.record.ReviewRecordDTO;
import com.example.survey.dto.record.SubmitRecordDTO;
import com.example.survey.entity.Project;
import com.example.survey.entity.Record;
import com.example.survey.entity.Respondent;
import com.example.survey.entity.User;
import com.example.survey.entity.inner.Operation;
import com.example.survey.enumeration.RecordStateEnum;
import com.example.survey.enumeration.RespondentStateEnum;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.ProjectException;
import com.example.survey.exception.RecordException;
import com.example.survey.exception.RespondentException;
import com.example.survey.exception.UserException;
import com.example.survey.service.RecordService;
import com.example.survey.vo.RecordVO;
import com.example.survey.vo.inner.OperationInfo;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @author Pope
 */
@Log4j2
@Service
public class RecordServiceImpl implements RecordService {


    @Autowired
    private RecordDao recordDao;

    @Autowired
    private RespondentDao respondentDao;

    @Autowired
    private UserDao userDao;

    @Autowired
    private ProjectDao projectDao;

    @Override
    public long countRecord(String userPhone, String projectName) {
        if (userPhone != null && !userDao.existUser(userPhone)) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        if (projectName != null && !projectDao.existProject(projectName)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        User user = userDao.selectUser(userPhone);
        Project project = projectDao.selectProject(projectName);
        return recordDao.countRecord(null, user, project, RecordStateEnum.UNDER_REVIEW.getValue(), null, null);
    }

    @Override
    public void reviewRecord(ReviewRecordDTO reviewRecordDTO) {
        if (!respondentDao.existRespondent(reviewRecordDTO.getIdNumber(), RespondentStateEnum.NOT_INVESTIGATED.getValue())) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        if (!userDao.existUser(reviewRecordDTO.getReviewerPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        if (!projectDao.existProject(reviewRecordDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(reviewRecordDTO.getProjectName());
        Respondent respondent = respondentDao.selectRespondent(reviewRecordDTO.getIdNumber(), project);
        User user = userDao.selectUser(reviewRecordDTO.getReviewerPhone());


        if (!recordDao.existRecord(respondent, project, RecordStateEnum.UNDER_REVIEW.getValue())) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        Record record = recordDao.getRecord(respondent, project, RecordStateEnum.UNDER_REVIEW.getValue());

        record.setState(reviewRecordDTO.getPass() ? RecordStateEnum.REVIEWED.getValue() : RecordStateEnum.NOT_PASS.getValue());
        record.setVersion(UUID.randomUUID().toString());
        Operation reviewOp = Operation.reviewOp(user, reviewRecordDTO.getMsg(), record.getVersion(), reviewRecordDTO.getPass());
        List<Operation> opList = record.getOperationList();
        opList.add(reviewOp);
        record.setOperationList(opList);

        recordDao.saveRecord(record);
    }

    @Override
    public void modifyRecord(ModifyRecordDTO modifyRecordDTO) {
        if (!respondentDao.existRespondent(modifyRecordDTO.getIdNumber(), RespondentStateEnum.NOT_INVESTIGATED.getValue())) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        if (!userDao.existUser(modifyRecordDTO.getUserPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        if (!projectDao.existProject(modifyRecordDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(modifyRecordDTO.getProjectName());
        Respondent respondent = respondentDao.selectRespondent(modifyRecordDTO.getIdNumber(), project);
        User user = userDao.selectUser(modifyRecordDTO.getUserPhone());


        if (!recordDao.existRecord(respondent, project, RecordStateEnum.UNDER_REVIEW.getValue())) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        Record record = recordDao.getRecord(respondent, project, RecordStateEnum.REVIEWED.getValue());

        //将原来的设为已归档 添加覆盖操作
        record.setState(RecordStateEnum.FILED.getValue());
        Operation coverOp = Operation.coverOp(user, record.getVersion());
        List<Operation> oldOpList = record.getOperationList();
        List<Operation> newOpList = record.getOperationList();
        oldOpList.add(coverOp);
        record.setOperationList(oldOpList);
        record.setVersion(UUID.randomUUID().toString());
        recordDao.saveRecord(record);

        //生成新的流调记录
        record.setId(null);
        record.setValues(modifyRecordDTO.getValues());
        record.setState(RecordStateEnum.UNDER_REVIEW.getValue());
        Operation modifyOp = Operation.modifyOp(user, modifyRecordDTO.getMsg(), record.getVersion());
        newOpList.add(modifyOp);
        record.setOperationList(newOpList);
        record.setVersion(UUID.randomUUID().toString());
        recordDao.saveRecord(record);
    }

    @Override
    public void createRecord(SubmitRecordDTO submitRecordDTO) {
        if (!respondentDao.existRespondent(submitRecordDTO.getIdNumber())) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        if (!userDao.existUser(submitRecordDTO.getUserPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        if (!projectDao.existProject(submitRecordDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(submitRecordDTO.getProjectName());
        Respondent respondent = respondentDao.selectRespondent(submitRecordDTO.getIdNumber(), project);
        User user = userDao.selectUser(submitRecordDTO.getUserPhone());

        //设置调查对象为已调查
        respondent.setState(RespondentStateEnum.INVESTIGATED.getValue());
        respondentDao.saveRespondent(respondent);

        Record record = recordDao.getRecord(respondent, project, RecordStateEnum.UNDER_REVIEW.getValue());
        if (record == null) {
            record = recordDao.getRecord(respondent, project, RecordStateEnum.REVIEWED.getValue());
        }
        if (record != null) {
            //存在旧纪录，覆盖
            List<Operation> oldOpList = record.getOperationList();
            List<Operation> newOpList = record.getOperationList();
            record.setState(RecordStateEnum.FILED.getValue());
            Operation coverOp = Operation.coverOp(user, record.getVersion());
            oldOpList.add(coverOp);
            record.setOperationList(oldOpList);
            record.setVersion(UUID.randomUUID().toString());
            recordDao.saveRecord(record);

            //插入新记录
            record.setId(null);
            record.setValues(submitRecordDTO.getValues());
            record.setState(RecordStateEnum.UNDER_REVIEW.getValue());
            Operation submitOp = Operation.submitOp(user, submitRecordDTO.getMsg(), record.getVersion());
            newOpList.add(submitOp);
            record.setOperationList(newOpList);
            record.setVersion(UUID.randomUUID().toString());
        } else {
            record = new Record();
            record.setRespondent(respondent);
            record.setUser(user);
            record.setValues(submitRecordDTO.getValues());
            record.setState(RecordStateEnum.UNDER_REVIEW.getValue());
            record.setVersion(UUID.randomUUID().toString());
            List<Operation> opList = new ArrayList<>();
            Operation submitOp = Operation.submitOp(user, submitRecordDTO.getMsg(), record.getVersion());
            opList.add(submitOp);
            record.setOperationList(opList);
            record.setProject(project);
        }
        recordDao.saveRecord(record);
    }

    @Override
    public long countRecord(String idNumber, String userPhone, String projectName, String state, String version, String questionnaireNumber) {
        if (idNumber != null && !respondentDao.existRespondent(idNumber)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        if (userPhone != null && !userDao.existUser(userPhone)) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        if (projectName != null && !projectDao.existProject(projectName)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(projectName);
        Respondent respondent = respondentDao.selectRespondent(idNumber, project);
        User user = userDao.selectUser(userPhone);


        return recordDao.countRecord(respondent, user, project, state, version, questionnaireNumber);
    }

    @Override
    public List<RecordVO> listRecordLimit(String idNumber, String userPhone, String projectName, String state, String version, String questionnaireNumber, int currentPage, int pageSize) {
        if (idNumber != null && !respondentDao.existRespondent(idNumber)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        if (userPhone != null && !userDao.existUser(userPhone)) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        if (projectName != null && !projectDao.existProject(projectName)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(projectName);
        Respondent respondent = respondentDao.selectRespondent(idNumber, project);
        User user = userDao.selectUser(userPhone);


        List<Record> recordList = recordDao.listRecordLimit(respondent, user, project, state, version, questionnaireNumber, currentPage * pageSize, pageSize);
        return recordList.stream().map(record -> {
            RecordVO recordVO = new RecordVO();
            recordVO.setIdNumber(record.getRespondent().getIdNumber());
            recordVO.setUserPhone(record.getUser().getPhone());
            recordVO.setProjectName(record.getProject().getName());
            recordVO.setOperationInfoList(record.getOperationList().stream().map(op -> {
                OperationInfo operationInfo = new OperationInfo();
                operationInfo.setType(op.getType());
                operationInfo.setTime(op.getTime());
                operationInfo.setUserPhone(op.getUser().getPhone());
                operationInfo.setMsg(op.getMsg());
                operationInfo.setResult(op.getResult());
                operationInfo.setVersion(op.getVersion());
                return operationInfo;
            }).collect(Collectors.toList()));
            recordVO.setVersion(record.getVersion());
            recordVO.setState(record.getState());
            return recordVO;
        }).collect(Collectors.toList());
    }

    @Override
    public Map<String, Object> getRecordValues(String projectName, String idNumber, String version) {
        if (!projectDao.existProject(projectName)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        if (!respondentDao.existRespondent(idNumber)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Project project = projectDao.selectProject(projectName);
        Respondent respondent = respondentDao.selectRespondent(idNumber, project);
        Record record = recordDao.selectRecord(respondent, project, version);
        if (record == null) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        return record.getValues();
    }


}
